/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.console.security;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

/**
 * The Nova Access List allow to map user profiles with user rights keys. It's
 * possible to set permissions, remove or consult if a user has the rights for a
 * given manipulation key
 * 
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 */

public class NovaACL {

  /* the access list allow with a profile to check the permissions */
  private Map<String, Collection<String>> accessList;
  private ProfileManager profileManager;

  /**
   * Create a new NovaACL
   * 
   * @param profileManager the profil manager
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public NovaACL(ProfileManager profileManager) {
    this.accessList = new HashMap<String, Collection<String>>();
    this.profileManager = profileManager;
  }

  /* ===== access list methods ===== */
  /**
   * check a permission
   * 
   * @param userName name of the user
   * @param key the permission key to check
   * @return true if the user has got the permission for the given key
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public boolean hasPermission(String userName, String key) {
    boolean hasPermission = false;
    Collection<String> profiles = this.profileManager.getProfiles(userName);
    Iterator<String> iter = profiles.iterator();
    while (iter.hasNext()) {
      String profile = iter.next();
      if (this.accessList.containsKey(profile)) {
        hasPermission = hasPermission || this.accessList.get(profile).contains(key);
      }
    }
    return hasPermission;
  }

  /**
   * Give a permission to a profile for a given key
   * 
   * @param profile a profile
   * @param key a permission key
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public void givePermission(String profile, String key) {
    if (!this.accessList.containsKey(profile)) {
      this.accessList.put(profile, new HashSet<String>());
    }
    this.accessList.get(profile).add(key);
  }

  /**
   * Give permissions to a profile for a set of key
   * 
   * @param profile a profile
   * @param keys a set of permission key
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo check if this method is usefull
   */
  public void givePermissions(String profile, Collection<String> keys) {
    if (!this.accessList.containsKey(profile)) {
      this.accessList.put(profile, new HashSet<String>());
    }
    this.accessList.get(profile).addAll(keys);
  }

  /**
   * Remove permissions to a profile for a set of key
   * 
   * @param profile a profile
   * @param keys a set of permission key
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo check if this method is usefull
   */
  public void removePermissions(String profile, Collection<String> keys) {
    if (this.accessList.containsKey(profile)) {
      this.accessList.get(profile).removeAll(keys);
    }
  }

  /**
   * Persists access list in JCR
   * 
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo implement this method
   * @todo check if this method is usefull
   */
  public void commit() {
    // TODO : implement the jcr persistance of the access list
  }
}
